package com.github.polpetta.mezzotre.telegram.callbackquery;

import com.github.polpetta.mezzotre.i18n.TemplateContentGenerator;
import com.github.polpetta.mezzotre.orm.model.CallbackQueryContext;
import com.github.polpetta.mezzotre.orm.model.query.QCallbackQueryContext;
import com.github.polpetta.mezzotre.util.UUIDGenerator;
import com.github.polpetta.types.json.CallbackQueryMetadata;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.model.request.InlineKeyboardButton;
import com.pengrad.telegrambot.model.request.InlineKeyboardMarkup;
import com.pengrad.telegrambot.model.request.ParseMode;
import com.pengrad.telegrambot.request.BaseRequest;
import com.pengrad.telegrambot.request.EditMessageText;
import com.pengrad.telegrambot.request.SendMessage;
import java.util.NoSuchElementException;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;
import org.slf4j.Logger;

/**
 * SelectLanguageTutorial event is related to support locale change for a particular chat when the
 * interaction with a new user starts
 *
 * @author Davide Polonio
 * @since 1.0
 */
@Singleton
public class SelectLanguageTutorial implements Processor {

  public static final String EVENT_NAME = "selectLanguageTutorial";
  private final Executor threadPool;
  private final TemplateContentGenerator templateContentGenerator;
  private final Logger log;
  private final UUIDGenerator uuidGenerator;

  @Inject
  public SelectLanguageTutorial(
      @Named("eventThreadPool") Executor threadPool,
      TemplateContentGenerator templateContentGenerator,
      Logger log,
      UUIDGenerator uuidGenerator) {
    this.threadPool = threadPool;
    this.templateContentGenerator = templateContentGenerator;
    this.log = log;
    this.uuidGenerator = uuidGenerator;
  }

  @Override
  public String getEventName() {
    return EVENT_NAME;
  }

  @Override
  public CompletableFuture<Optional<BaseRequest<?, ?>>> process(
      CallbackQueryContext callbackQueryContext, Update update) {
    return CompletableFuture.supplyAsync(
            () ->
                Util.extractChat(callbackQueryContext, update)
                    .map(
                        tgChat -> {
                          tgChat.setLocale(
                              (String)
                                  callbackQueryContext
                                      .getFields()
                                      .getAdditionalProperties()
                                      .getOrDefault(
                                          Field.SelectLanguageTutorial.NewLanguage.getName(),
                                          tgChat.getLocale()));
                          tgChat.save();
                          log.trace(
                              "Locale for chat "
                                  + tgChat.getId()
                                  + " is now set in "
                                  + tgChat.getLocale());
                          return tgChat;
                        })
                    .orElseThrow(
                        () ->
                            new NoSuchElementException(
                                "Unable to find telegram chat "
                                    + Double.valueOf(
                                            callbackQueryContext.getFields().getTelegramChatId())
                                        .longValue()
                                    + " in the database")),
            threadPool)
        .thenApplyAsync(
            // If we are here then we're sure there is at least a chat associated with this callback
            tgChat -> {
              final String message =
                  templateContentGenerator.mergeTemplate(
                      velocityContext ->
                          velocityContext.put("hasHelpBeenShown", tgChat.getHasHelpBeenShown()),
                      tgChat.getLocale(),
                      "/template/telegram/selectLanguageTutorial.vm");

              log.trace("SelectLanguageTutorial event - message to send back: " + message);

              final String callBackGroupToDelete = callbackQueryContext.getEntryGroup();
              final int delete =
                  new QCallbackQueryContext().entryGroup.eq(callBackGroupToDelete).delete();
              log.trace(
                  "Deleted "
                      + delete
                      + " entries regarding callback group "
                      + callBackGroupToDelete);

              final Optional<Integer> messageId = Util.extractMessageId(update);
              BaseRequest<?, ?> baseRequest;
              Optional<InlineKeyboardMarkup> helpButton = Optional.empty();
              if (!tgChat.getHasHelpBeenShown()) {
                // Add a button to show all the possible commands
                final String showMeTutorialString =
                    templateContentGenerator.getString(
                        tgChat.getLocale(),
                        "selectLanguageTutorial.showMeTutorialInlineKeyboardButtonName");

                final CallbackQueryMetadata callbackQueryMetadata =
                    new CallbackQueryMetadata.CallbackQueryMetadataBuilder()
                        .withEvent(ShowHelp.EVENT_NAME)
                        .withTelegramChatId(tgChat.getId())
                        .build();

                final CallbackQueryContext callbackData =
                    new CallbackQueryContext(
                        uuidGenerator.generateAsString(),
                        uuidGenerator.generateAsString(),
                        callbackQueryMetadata);
                callbackData.save();

                helpButton =
                    Optional.of(
                        new InlineKeyboardMarkup(
                            new InlineKeyboardButton(showMeTutorialString)
                                .callbackData(callbackData.getId())));
              }

              if (messageId.isPresent()) {
                log.trace("message id is present - editing old message");
                final EditMessageText editMessageText =
                    new EditMessageText(tgChat.getId(), messageId.get(), message)
                        .parseMode(ParseMode.Markdown);
                helpButton.ifPresent(editMessageText::replyMarkup);
                baseRequest = editMessageText;
              } else {
                log.trace("no message id - sending a new message");
                final SendMessage sendMessage =
                    new SendMessage(tgChat.getId(), message).parseMode(ParseMode.Markdown);
                helpButton.ifPresent(sendMessage::replyMarkup);
                baseRequest = sendMessage;
              }

              return Optional.of(baseRequest);
            },
            threadPool);
  }
}
